From 45de723e28ba3caf19ff8702344bbae1629c5a7a Mon Sep 17 00:00:00 2001
From: unknown <Solutions@.(none)>
Date: Wed, 14 Jul 2010 11:34:13 +0200
Subject: [PATCH 2/2] Seek support in imem access module

---
 modules/access/imem.c |   31 +++++++++++++++++++++++++++++--
 1 files changed, 29 insertions(+), 2 deletions(-)

diff --git a/modules/access/imem.c b/modules/access/imem.c
index 6ee4256..b0aeadd 100644
--- a/modules/access/imem.c
+++ b/modules/access/imem.c
@@ -115,6 +115,10 @@ static const char *cat_texts[] = {
 #define RELEASE_LONGTEXT N_(\
     "Address of the release callback function")
 
+#define SEEK_TEXT N_("Seek function")
+#define SEEK_LONGTEXT N_(\
+    "Address of the seek callback function")
+
 #define SIZE_TEXT N_("Size")
 #define SIZE_LONGTEXT N_(\
 	"Size of stream in bytes")
@@ -131,6 +135,8 @@ vlc_module_begin()
         change_volatile()
     add_string ("imem-release", "0", NULL, RELEASE_TEXT, RELEASE_LONGTEXT, true)
         change_volatile()
+    add_string ("imem-seek", "0", NULL, SEEK_TEXT, SEEK_LONGTEXT, true)
+        change_volatile()
     add_string ("imem-cookie", NULL, NULL, COOKIE_TEXT, COOKIE_LONGTEXT, true)
         change_volatile()
         change_safe()
@@ -201,12 +207,14 @@ typedef int  (*imem_get_t)(void *data, const char *cookie,
                            int64_t *dts, int64_t *pts, unsigned *flags,
                            size_t *, void **);
 typedef void (*imem_release_t)(void *data, const char *cookie, size_t, void *);
+typedef int (*imem_seek_t)(void * data, uint64_t);
 
 /*****************************************************************************
  * Local prototypes	
  *****************************************************************************/
 
 /* */
+static int Seek(access_t *, uint64_t);
 static block_t *Block(access_t *);
 static int ControlAccess(access_t *, int, va_list);
 
@@ -218,6 +226,7 @@ typedef struct {
     struct {
         imem_get_t      get;
         imem_release_t  release;
+        imem_seek_t     seek;
         void           *data;
         char           *cookie;
     } source;
@@ -269,6 +278,11 @@ static int OpenCommon(vlc_object_t *object, imem_sys_t **sys_ptr, const char *ps
         sys->source.release = (imem_release_t)(intptr_t)strtoll(tmp, NULL, 0);
     free(tmp);
 
+    tmp = var_InheritString(object, "imem-seek");
+    if (tmp)
+        sys->source.seek = (imem_seek_t)(intptr_t)strtoll(tmp, NULL, 0);
+    free(tmp);
+
     if (!sys->source.get || !sys->source.release) {
         msg_Err(object, "Invalid get/release function pointers");
         free(sys);
@@ -321,7 +335,7 @@ static int OpenAccess(vlc_object_t *object)
     access->pf_control = ControlAccess;
     access->pf_read    = NULL;
     access->pf_block   = Block;
-    access->pf_seek    = NULL;
+    access->pf_seek    = (sys->source.seek == NULL ? NULL : Seek);
     access->p_sys      = (access_sys_t*)sys;
 
     // we've got size as string, becouse we have no (u)int64_t var functions
@@ -354,7 +368,7 @@ static int ControlAccess(access_t *access, int i_query, va_list args)
     case ACCESS_CAN_SEEK:
     case ACCESS_CAN_FASTSEEK: {
         bool *b = va_arg( args, bool* );
-        *b = false;
+        *b = (sys->source.seek != NULL);
         return VLC_SUCCESS;
     }
     case ACCESS_CAN_PAUSE:
@@ -384,6 +398,19 @@ static int ControlAccess(access_t *access, int i_query, va_list args)
 }
 
 /**
+ * It seeks through the stream using the seek() callback
+ */
+static int Seek(access_t *access, uint64_t seekTo)
+{
+    imem_sys_t * sys = (imem_sys_t*)access->p_sys;
+
+    if(sys->source.seek)
+        return sys->source.seek(sys->source.data, seekTo);
+
+    return VLC_EGENERIC;
+}
+
+/**
  * It retreives data using the get() callback, copies them,
  * and then release them using the release() callback.
  */
-- 
1.7.1.1923.gc1c7c

